//////////////////////////////////////////////////////////////////////
// Sniffer.cpp
// -------------------------------------------------------------------
// Very basic sniffer module.
//
// <thohell@home.se>
//////////////////////////////////////////////////////////////////////
#include "..\ClientCore.cpp"

// Macros
inline char * booltostr( bool x )
{
	return (x ? "on" : "off");
}

inline char * booltostrb( bool x )
{
	return (x ? "true" : "false");
}

// Functions as they appear in the source...
BOOL PRIVATE OnGameCommandShow(char** argv, int argc);
BOOL PRIVATE OnGameCommandHide(char** argv, int argc);
BOOL PRIVATE OnGameCommandList(char** argv, int argc);
BOOL PRIVATE OnGameCommandLoad(char** argv, int argc);
BOOL PRIVATE OnGameCommandSave(char** argv, int argc);			//1-18-2002 - Modified - DFlame
BOOL PRIVATE OnGameCommandClear(char** argv, int argc);			//1-18-2002 - Modified - DFlame
BOOL PRIVATE isSetBit(BYTE* mask, int bit);
BOOL PRIVATE setBit(BYTE* mask, int bit, BOOL set);
BOOL PRIVATE OnGameCommandFileLog(char **argv, int argc);		//1-18-2002 - Modified - DFlame

BOOL PRIVATE OnGameCommandMark(char **argv, int argc);			//1-18-2002 - DFlame
BOOL PRIVATE OnGameCommandLogPause(char **argv, int argc);		//1-18-2002 - DFlame
BOOL PRIVATE OnGameCommandOptions(char **argv, int argc);		//1-18-2002 - DFlame
BOOL PRIVATE OnGameCommandDecode(char **argv, int argc);		//1-18-2002 - DFlame

BOOL PRIVATE OnGameCommandOutput(char **argv, int argc);

void PRIVATE GetTimeString( char *pszBuffer, char **argv, int argc ); //1-18-2002 - DFlame

// Some global stuff
FINGERPRINTSTRUCT fps;

LPSTR	RecvName[257];
LPSTR	SendName[257];

char	CurrentSection[64];

BYTE	RecvMask[32];
BYTE	SendMask[32];

//Globals
bool	log_file_set = false;
bool	output = true;
bool	bPaused = false;
bool	bOverwrite = false;
bool	bAutoSave = false;
bool	bDisplayChar = true;
bool	bDecode = false;
bool	bAlphaOnly = true;
bool	bShowDate = false;
bool	bShowTime = true;
bool	b12h = true;

LPSTR	log_file;
FILE	*pLogFile = NULL;

//////////////////////////////////////////////////////////////////////
// CLIENTINFO
// -------------------------------------------------------------------
// You *need* this, don't ask why :)
//
// 0.01 2001-10-14 First version
// 0.02 2001-11-21 gototown added by druttis (/Blackpower Ugh?)
// 0.03 2002-01-14 log added by Blackpower
// 0.04 2002-01-14 output added by Blackpower
// 0.05 2002-01-17 Check before Autosave, Pause, char Display,
//					Mark, Decode Tag, and TimeStamp by DFlame
// 0.10 2002-01-18 Felt my changes are bigger than I first thought - DFlame
//					FileLogging routine just leaves handle open til the end now
//					Loads options on start, Can save options
// 0.11 2002-10-19 Debugged Alpha I think everything works now
//////////////////////////////////////////////////////////////////////
DWORD	ModuleVersion=MAKELONG(0,5);
char	ModuleAuthor[]="thohell, druttis, Blackpower, & DFlame";
char	ModuleWebsite[]="thohell.d2network.com";
char	ModuleDescription[]="Sniffer module - Very simple D2 sniffer";
char	ModuleEmail[]="thohell@home.se, druttis@darkface.pp.se, blackpower@code-monkey.co.uk,\n"
						"DFlame666@pacbell.net";

//////////////////////////////////////////////////////////////////////
// MODULECOMMANDSTRUCT ModuleCommands[]
// -------------------------------------------------------------------
// To add your own commands, just add them to this list like:
//
//	{
//		"name"			// Name of the command
//		pFunction		// The function that handles command
//		"usage"			// Help text for the command
//	}
// 
// Functions should use following convention...
// BOOL PRIVATE OnGameCommandName(char** argv, int argc)
// ...and return TRUE on success or FALSE to show usage of the command.
//
// To use your commands in the game, just type 
// .modulename <command> [arguments]
//////////////////////////////////////////////////////////////////////

MODULECOMMANDSTRUCT ModuleCommands[]=
{
	{	// The 'help' command is implemented in ClientCore.cpp and 
		// should not need to be changed.
		"help",
		OnGameCommandHelp,
		"helpc0 List commands available in this module.\n"
		"<command> helpc0 Shows detailed help for <command> in this module."
	},
	{
		"show",
		OnGameCommandShow,
		"show <R|S> <id> [id] ... [id]c0 Shows <'R'eceived|'S'ent> packets with packet id <id>."
	},
	{
		"hide",
		OnGameCommandHide,
		"hide <R|S> <id> [id] ... [id]c0 Hides <'R'eceived|'S'ent> packets with packet id <id>."
	},
	{
		"list",
		OnGameCommandList,
		"listc0 Lists all packets id's that will show."
	},
	{	"load",
		OnGameCommandLoad,
		"load [section]c0 Loads packet-filter in [section] from sniffer.ini."
	},
	{	"save",		// Modified to not overwrite the originals - DFlame
		OnGameCommandSave,
		"save [section]c0 Saves current packet-filter to [section] in sniffer.ini."
	},
	{	"clear",	// Modified to turn off Autosave - DFlame
		OnGameCommandClear,
		"clearc0 Removes all filters (Show nothing)."
	},
	{	//1-18-2002 - Highly Modified - DFlame
		"logfile",
		OnGameCommandFileLog,
		"logfile <file>c0 Logs packets to <file>.txt in your D2 dir.\n"
		"c4logfile stopc0 Stops the logging to <file>.\n"
		"+/-overwritec0 Log File write mode."
	},
	{	//1-18-2002 - DFlame
		"mark",
		OnGameCommandMark,
		"mark <your tag>c0 Marks <your tag> at the current file writing position.\n"
		"c4$d $tc0 Translated to Date and time tags, can be condensed to $dt."
	},
	{	//1-18-2002 - DFlame
		"logpause",
		OnGameCommandLogPause,
		"logpausec0 Pauses and unpauses writing to the log file."
	},
	{	//1-18-2002 - DFlame
		"options",
		OnGameCommandOptions,
		"options <[+/-]date ?time ?12h ?char ?alpha ?setname ?overwrite ?pause ?autosave save>c0 Sets misc options.\n"
		"?date for log date stamp, ?time for log time stamp, ?char for char display of data after packet data,\n"
		"?alpha for filtering mode of char display, will convert only letters, ?setname - packet name decoding.\n"
		"?overwrite for Log file right mode, ?autosave for filter set saving, save - Save options"
	},
	{	//1-18-2002 - Packet Name Decoding
		"setname",
		OnGameCommandDecode,
		"setname <r|s|save|load> <Packet #> [Decode Name]c0 Gives that Packet number a name.\n"
		"load save - File related."
	},
	{
		"output",
		OnGameCommandOutput,
		"outputc0 Toggles display output of packets on/off"
	},

	{NULL}	// No more commands
};

//////////////////////////////////////////////////////////////////////
// OnClientStart
// -------------------------------------------------------------------
// Runs *once* before client is loaded. Return FALSE to prevent from
// loading.
//////////////////////////////////////////////////////////////////////
BOOL EXPORT OnClientStart()
{
	// Initiate list
	char *argv[3];
	char argc=3;
	argv[0]="";
	argv[1]="load";
	argv[2]="default";
	OnGameCommandLoad(argv, argc);

	// New Options Loading Code - DFlame
	bAutoSave = ( strcmp( server->GetHackProfileString("sniffer", "Options", "Autosave"), "true" ) == 0 ? true : false );
	bOverwrite = ( strcmp( server->GetHackProfileString("sniffer", "Options", "Overwrite"), "true" ) == 0 ? true : false );
	bDisplayChar = ( strcmp( server->GetHackProfileString("sniffer", "Options", "DisplayChar"), "true" ) == 0 ? true : false );
	bAlphaOnly = ( strcmp( server->GetHackProfileString("sniffer", "Options", "Alpha"), "true" ) == 0 ? true : false );
	bShowDate = ( strcmp( server->GetHackProfileString( "sniffer", "Options", "TagDate"), "true" ) == 0 ? true : false );
	bShowTime = ( strcmp( server->GetHackProfileString( "sniffer", "Options", "TagTime"), "true" ) == 0 ? true : false );
	bDecode = ( strcmp( server->GetHackProfileString("sniffer", "Decode", "Decode"), "true" ) == 0 ? true : false );
	b12h = ( strcmp( server->GetHackProfileString("sniffer", "Options", "12h"), "true" ) == 0 ? true : false );


	for( int i=0; i < 256; i++ )
	{
		delete RecvName[i];
		delete SendName[i];
		RecvName[i] = NULL;
		SendName[i] = NULL;
	}
	
	return TRUE;
}

//////////////////////////////////////////////////////////////////////
// OnClientStop
// -------------------------------------------------------------------
// Runs *once* before client is unloaded. Return value should be TRUE,
// but it is not used at this time.
//////////////////////////////////////////////////////////////////////
BOOL EXPORT OnClientStop()
{
	char *argv[2];
	char argc=2;

	if( bAutoSave )					// Options Related - DFlame
	{
		argv[0]="";
		argv[1]="autosave";
		OnGameCommandSave(argv, argc);
	}

	for( int i=0; i < 256; i++ )
	{
		delete RecvName[i];
		delete SendName[i];
	}
	delete log_file;
	return TRUE;
}

//////////////////////////////////////////////////////////////////////
// OnClientCommandShow
// -------------------------------------------------------------------
//////////////////////////////////////////////////////////////////////
BOOL PRIVATE OnGameCommandShow(char** argv, int argc)
{
	// If command line is shorter than 4 tokens, something is wrong
	if (argc<4) return FALSE;

	bool r;

	if (!stricmp(argv[2], "r"))
		r = TRUE;
	else if (!stricmp(argv[2], "s"))
		r = FALSE;
	else
		return FALSE;

	for (int i=3; i!=argc; i++) {
		char* x = "";
		DWORD v = (strtoul(argv[i], &x, 0x10) & 0xff);
		if (r)
			setBit(RecvMask, v, TRUE);
		else
			setBit(SendMask, v, TRUE);
	}	

	server->GamePrintInfo("Ok");
	return TRUE;
}

//////////////////////////////////////////////////////////////////////
// OnClientCommandHide
// -------------------------------------------------------------------
//////////////////////////////////////////////////////////////////////
BOOL PRIVATE OnGameCommandHide(char** argv, int argc)
{
	// If command line is shorter than 4 tokens, something is wrong
	if (argc<4) return FALSE;

	bool r;

	if (!stricmp(argv[2], "r"))
		r = TRUE;
	else if (!stricmp(argv[2], "s"))
		r = FALSE;
	else
		return FALSE;

	for (int i=3; i!=argc; i++) {
		char* x = "";
		DWORD v = (strtoul(argv[i], &x, 0x10) & 0xff);
		if (r)
			setBit(RecvMask, v, FALSE);
		else
			setBit(SendMask, v, FALSE);
	}	

	server->GamePrintInfo("Ok");
	return TRUE;
}

//////////////////////////////////////////////////////////////////////
// OnClientCommandList
// -------------------------------------------------------------------
//////////////////////////////////////////////////////////////////////
BOOL PRIVATE OnGameCommandList(char** argv, int argc)
{
	// If command line is not 2 tokens, something is wrong
	if (argc != 2) return FALSE;

	char	s[8][110], r[8][110];
	int		rC = 0, sC = 0;
	int		i;

	for	(i = 0; i < 8;i++)
	{
		sprintf(r[i], "c8RECV: ");
		sprintf(s[i], "c9SENT: ");
	}

	server->GamePrintInfo("Packets that are currently visible:");

	// Format restult
	for (i = 0; i < 256; i++)
	{
		if (isSetBit(RecvMask, i)) {
			sprintf(r[rC], "%s %.2x", r[rC], i);
			if (strlen(r[rC]) > 104)
				rC++;
		}

		if (isSetBit(SendMask, i)) {
			sprintf(s[sC], "%s %.2x", s[sC], i);
			if (strlen(s[sC]) > 104)
				sC++;
		}
	}

	// Print result
	for (i=0; i < 8; i++) {
		if (strlen(r[i]) > 10)
			server->GamePrintInfo(r[i]);
	}

	for (i=0; i < 8; i++) {
		if (strlen(s[i]) > 10)
			server->GamePrintInfo(s[i]);
	}

	return TRUE;
}

//////////////////////////////////////////////////////////////////////
// OnClientCommandLoad
// -------------------------------------------------------------------
//////////////////////////////////////////////////////////////////////
BOOL PRIVATE OnGameCommandLoad(char** argv, int argc)
{
	// If command line is not 2 or 3 tokens, something is wrong
	if ((argc < 2) && (argc > 3))return FALSE;

	char t[128];

	if (argc == 3)
		strcpy(CurrentSection, argv[2]);

	sprintf(t, "Loading packet-filter from section: c4[%s]",CurrentSection);
	server->GamePrintInfo(t);

	char* r=server->GetHackProfileString("sniffer", CurrentSection, "RECV");
	char* s=server->GetHackProfileString("sniffer", CurrentSection, "SENT");

	int c = 0;
	char* x = "";
	for (int i = 0; i < 32; i++) {
		r[c + 2] = 0;
		s[c + 2] = 0;
		RecvMask[i] = (strtoul(&r[c], &x, 0x10) & 0xff);
		SendMask[i] = (strtoul(&s[c], &x, 0x10) & 0xff);
		c += 3;
	}

	delete r, s;

	return TRUE;
}

//////////////////////////////////////////////////////////////////////
// OnClientCommandSave
// -------------------------------------------------------------------
//////////////////////////////////////////////////////////////////////
BOOL PRIVATE OnGameCommandSave(char** argv, int argc)
{
	// If command line is not 2 or 3 tokens, something is wrong
	if ((argc < 2) && (argc > 3))return FALSE;

	char t[1024];

	if (argc == 3)
	{
		strcpy(CurrentSection, argv[2]);
		if( strcmp( CurrentSection, "show-all" ) == 0 )
			return FALSE;
		if( strcmp( CurrentSection, "sent-all" ) == 0 )
			return FALSE;
		if( strcmp( CurrentSection, "recv-all" ) == 0 )
			return FALSE;
		//if( strcmp( CurrentSection, "default" ) )
		//	return FALSE;
	}

	sprintf(t, "Saving packet-filter to section: c4[%s]", CurrentSection);
	server->GamePrintInfo(t);

	char s[257], r[257];
	r[256] = s[256] = 0;

	sprintf(r, "%.2x", RecvMask[0]);
	sprintf(s, "%.2x", SendMask[0]);
	for (int i = 1; i < 32; i++)
	{
		sprintf(r, "%s %.2x", r, RecvMask[i]);
		sprintf(s, "%s %.2x", s, SendMask[i]);
	}

	server->SetHackProfileString("sniffer", CurrentSection, "RECV", r);
	server->SetHackProfileString("sniffer", CurrentSection, "SENT", s);

	return TRUE;
}

//////////////////////////////////////////////////////////////////////
// OnClientCommandClear
// -------------------------------------------------------------------
//////////////////////////////////////////////////////////////////////
BOOL PRIVATE OnGameCommandClear(char** argv, int argc)
{
	// If command line is not 2 tokens, something is wrong
	if (argc != 2) return FALSE;
	
	for (int i = 0; i < 32; i++) {
		RecvMask[i] = 0;
		SendMask[i] = 0;
	}

	bAutoSave = false;

	server->GamePrintInfo("All packet-filters has been cleared. No packets are visible. AutoSave turned off.");

	return TRUE;
}

//////////////////////////////////////////////////////////////////////
// OnGamePacketBeforeSent
// -------------------------------------------------------------------
//////////////////////////////////////////////////////////////////////
DWORD EXPORT OnGamePacketBeforeSent (BYTE * aPacket, DWORD aLen)
{
	// Filter
	if (!isSetBit (SendMask, aPacket[0]))
		return aLen;
	//Changed for log and output
	char swi[1024];
	char t[1024];

	if (log_file_set || output)
	{
		sprintf (t, " ");
		for (int i = 0; i < aLen; i++)
			sprintf (t, "%s%.2x ", t, aPacket[i]);

		// Packet Name Decoding
		i = aPacket[0];
		if( bDecode )
			sprintf( t, "%s  %s", t, SendName[i] );
		
		if( bDisplayChar )			// Display Char version or Packet Data
		{
			if( bAlphaOnly )		// Alphabet Only
			{
				sprintf( t, "%s  ", t );
				for (i = 1; i < aLen; i++)
				{
					if( (aPacket[i] > 0x40 && aPacket [i] < 0x5b) ||
						(aPacket[i] > 0x60 && aPacket [i] < 0x7b) )
						sprintf( t, "%s%c", t, (char)aPacket[i] );
					else sprintf( t, "%s0", t );
				}
			}
			else					// All Displayable Chars
			{
				sprintf( t, "%s  ", t );
				for (i = 1; i < aLen; i++)
				{
					if( (aPacket[i] > 0x20 && aPacket [i] <= 0xff) )
						sprintf( t, "%s%c", t, (char)aPacket[i] );
					else sprintf( t, "%s0", t );
				}
			}
		}
	}
	
	if ( output )
	{
		sprintf (swi, "c9SENTc0:%s", t);
		server->GamePrintString (swi);
	}
	if (log_file_set && !bPaused)
	{
		
/*		FILE *stream;		// Cutting Down on opens and closes
		if ((stream = fopen (log_file, "a")) == NULL)
		{
			sprintf (t, "Unable to append c8%sc8", log_file);
			server->GamePrintError (t);
			log_file_set = false;
			return aLen;
		}
*/		
		
		char *szTmpargv[3];		// New Time Stamp code
		int nTmpargc = 0;
		
		if( bShowDate )
		{
			szTmpargv[0] = "date";
			nTmpargc++;
		}
		if( bShowTime )
		{
			szTmpargv[nTmpargc] = "time";
			nTmpargc++;
		}
		if( b12h )
		{
			szTmpargv[nTmpargc] = "12h";
			nTmpargc++;
		}

		GetTimeString( swi, szTmpargv, nTmpargc );		// Timestamp
		sprintf (swi, "%s  SENT:%s", swi, t);
		fprintf ( pLogFile, "%s\n", swi);
		
		//fclose (stream);		//Leave open
		//delete stream;
	}

	return aLen;
}

//////////////////////////////////////////////////////////////////////
// OnGamePacketBeforeReceived
// -------------------------------------------------------------------
//////////////////////////////////////////////////////////////////////
DWORD EXPORT OnGamePacketBeforeReceived (BYTE * aPacket, DWORD aLen)
{
	// Filter
	if (!isSetBit (RecvMask, aPacket[0]))
		return aLen;
	//Changed for log and output

	char swi[1024];
	char t[1024];
	int i;
	if (log_file_set || output)
	{
		sprintf (t, " ");
		for (i = 0; i < aLen; i++)
			sprintf (t, "%s %.2x", t, aPacket[i]);
		
		i = aPacket[0];
		if( bDecode )		// Packet Name Decoding
				sprintf( t, "%s  %s", t, RecvName[i] );
		
		if( bDisplayChar )		// Display char version of Packet data
		{
			if( bAlphaOnly )	// Alphabet only
			{
				sprintf( t, "%s  ", t );
				for (i = 1; i < aLen; i++)
				{
					if( (aPacket[i] > 0x40 && aPacket [i] < 0x5b) ||
						(aPacket[i] > 0x60 && aPacket [i] < 0x7b) )
						sprintf( t, "%s%c", t, (char)aPacket[i] );
					else sprintf( t, "%s0", t );
				}
			}
			else				// All displayable chars
			{
				sprintf( t, "%s  ", t );
				for (i = 1; i < aLen; i++)
				{
					if( (aPacket[i] > 0x20 && aPacket [i] <= 0xff) )
						sprintf( t, "%s%c", t, (char)aPacket[i] );
					else sprintf( t, "%s0", t );
				}
			}
		}
	}

	if( output )
	{
		sprintf ( swi, "c8RECVc0:%s", t );
		server->GamePrintString ( swi );
	}
	
	if( log_file_set && !bPaused )	// Modified for LogPause code
	{
/*		FILE *stream;		// I think opening and closing the file all the time is slow - DFlame
		if ((stream = fopen (log_file, "a+")) == NULL)		// If you don't close the Handle
		{													// It should close by itself
			sprintf (t, "Unable to append c8%sc0", log_file);
			server->GamePrintError (t);
			log_file_set = false;
			return aLen;
		}
*/		

		char *szTmpargv[3];		// New Time Stamp code
		int nTmpargc = 0;
		
		if( bShowDate )
		{
			szTmpargv[0] = "date";
			nTmpargc++;
		}
		if( bShowTime )
		{
			szTmpargv[nTmpargc] = "time";
			nTmpargc++;
		}
		if( b12h )
		{
			szTmpargv[nTmpargc] = "12h";
			nTmpargc++;
		}

		GetTimeString( swi, szTmpargv, nTmpargc );
		sprintf ( swi, "%s  RECV:%s", swi, t );
		fprintf ( pLogFile, "%s\n", swi ); // Changed for time stamp
		
		//fclose (stream);		// Left open
		//delete stream;
    }

	return aLen;
}

//////////////////////////////////////////////////////////////////////
// isBitSet
// -------------------------------------------------------------------
// Check if bit in mask is set
//////////////////////////////////////////////////////////////////////
BOOL PRIVATE isSetBit(BYTE* mask, int bit)
{
	if ((bit < 0) || (bit > 255))
			return FALSE;
	return ((mask[bit >> 3] & (1 << (bit & 0x7))) != 0 ? 1 : 0);
}

//////////////////////////////////////////////////////////////////////
// setBit
// -------------------------------------------------------------------
// Set or unsets a bit in mask
//////////////////////////////////////////////////////////////////////
BOOL PRIVATE setBit(BYTE* mask, int bit, BOOL set)
{
	if ((bit < 0) || (bit >255))
		return FALSE;
	if (set) {
		mask[bit >> 3] |= (1 << (bit & 0x7));
	} else {
		mask[bit >> 3] &= 0xff - (1 << (bit & 0x7));
	}
	return TRUE;
}

//////////////////////////////////////////////////////////////////////
// OnGameCommandLog
// -------------------------------------------------------------------
//////////////////////////////////////////////////////////////////////
BOOL PRIVATE
OnGameCommandFileLog (char **argv, int argc)
{
	char	szTemp[1024];
	//char	szTemp2[1024];
	char	*szTempargv[3] = { "date", "time", "12h" };
	int		nTempargc = 3;

	if (argc < 3 || argc > 4)
		return false;

	for( int i = 0; i < argc; i++ )
	{
		if ( (strcmp(argv[ i ], "stop") == 0) && log_file_set )
		{
			log_file_set = false;
			server->GamePrintInfo ("c8Logging has stoppedc0");
		
			GetTimeString( szTemp, szTempargv, nTempargc );
			fprintf( pLogFile, "\nLogging Stopped: %s\n", szTemp );

			fclose( pLogFile );
			delete log_file;

			return true;
		}
		if( strcmp( argv[ i ], "+overwrite" ) == 0 )
			bOverwrite = true;
		if( strcmp( argv[ i ], "-overwrite" ) == 0 )
			bOverwrite = false;
		if( strcmp( argv[ i ], "overwrite" ) == 0 )
			bOverwrite = !bOverwrite;
    }
	//FILE *stream;

	if( log_file_set )
	{
		log_file_set = false;
		fclose( pLogFile );
		delete log_file;
	}

	char szD2Dir[256];

	GetCurrentDirectory (255, szD2Dir);
	sprintf( szD2Dir, "%s\\logs\\", szD2Dir );

	log_file = new char[ strlen( szD2Dir ) + strlen( argv[2] ) + 6 ];
	sprintf ( log_file, "%s\\%s.txt", szD2Dir, argv[2] );
	
	if( bOverwrite )
	{
		if( !(pLogFile = fopen( log_file, "w" )) )
		{
			sprintf (szTemp, "Unable to create or openc8%sc0.", log_file);
			server->GamePrintError ( szTemp );
			
			delete log_file;
			return false;
		}
	}
	else
	{
		if( !(pLogFile = fopen( log_file, "a+" )) )
		{
			sprintf ( szTemp, "Unable to create or openc8%sc0.", log_file );
			server->GamePrintError ( szTemp );

			delete log_file;
			return false;
		}
		fprintf( pLogFile, "\n\n" );
	}


	GetTimeString( szTemp, szTempargv, nTempargc );
	fprintf( pLogFile, "Logging Started: %s \n", szTemp );

	sprintf( szTemp, "Logging to c8%sc0", log_file );
	server->GamePrintInfo ( szTemp );
	//fclose (stream);
	//delete stream;
	log_file_set = true;
	return true;
}


//////////////////////////////////////////////////////////////////////
// OnGameCommandMark
// -------------------------------------------------------------------
// Put a user entered Marker tag at file writing Position
//////////////////////////////////////////////////////////////////////
BOOL PRIVATE OnGameCommandMark(char **argv, int argc)			//1-18-2002 - DFlame
{
	char szTemp[1024];

	if( argc < 3 )
	{
		server->GamePrintInfo( "Give me something to mark down -.-" );
		return false;
	}

	if( !log_file_set )
	{
		server->GamePrintInfo( "Open a file to mark first." );
		return false;
	}

	sprintf( szTemp, "\n\n" );
	for( int i = 2; i < argc; i++ )
	{
		sprintf( szTemp, "%s%s", szTemp, argv[i] );
		if( i != argc )
			sprintf( szTemp, "%s ", szTemp, argv[i] );
	}
	sprintf( szTemp, "%s\n\n", szTemp );

	fprintf( pLogFile, szTemp );

	return true;
}


//////////////////////////////////////////////////////////////////////
// OnGameCommandLogPause
// -------------------------------------------------------------------
// Pause writing to file
//////////////////////////////////////////////////////////////////////
BOOL PRIVATE OnGameCommandLogPause(char **argv, int argc)			//1-18-2002 - DFlame
{
	if (argc != 2)
		return false;
	if (bPaused)
	{
		bPaused = false;
		server->GamePrintInfo ("Log Pause off.");
	}
	else
	{
		bPaused = true;
		server->GamePrintInfo ("Log Pause on.");
	}
	return true;
}


//////////////////////////////////////////////////////////////////////
// OnGameCommandOption
// -------------------------------------------------------------------
// Parse and set additional feature options
//////////////////////////////////////////////////////////////////////
BOOL PRIVATE OnGameCommandOptions(char **argv, int argc)		//1-18-2002 - DFlame
{
	char szTemp[1024];
	bool bSaveOptions = false;

	if( argc < 2 )
		return false;

	for( int i = 0; i < argc; i++ )
	{
		if( strcmp( argv[i], "help" ) == 0 )
			return false;
		if( strcmp( argv[i], "-date" ) == 0 )
			bShowDate = false;
		if( strcmp( argv[i], "+date" ) == 0 )
			bShowDate = true;
		if( strcmp( argv[i], "date" ) == 0 )
			bShowDate = !bShowDate;
		if( strcmp( argv[i], "-time" ) == 0 )
			bShowTime = false;
		if( strcmp( argv[i], "+time" ) == 0 )
			bShowTime = true;
		if( strcmp( argv[i], "time" ) == 0 )
			bShowTime = !bShowTime;
		if( strcmp( argv[i], "-12h" ) == 0 )
			b12h = false;
		if( strcmp( argv[i], "+12h" ) == 0 )
			b12h = true;
		if( strcmp( argv[i], "12h" ) == 0 )
			b12h = !b12h;
		if( strcmp( argv[i], "-char" ) == 0 )
			bDisplayChar = false;
		if( strcmp( argv[i], "+char" ) == 0 )
			bDisplayChar = true;
		if( strcmp( argv[i], "char" ) == 0 )
			bDisplayChar = !bDisplayChar;
		if( strcmp( argv[i], "-alpha" ) == 0 )
			bAlphaOnly = false;
		if( strcmp( argv[i], "+alpha" ) == 0 )
			bAlphaOnly = true;
		if( strcmp( argv[i], "alpha" ) == 0 )
			bAlphaOnly = !bAlphaOnly;
		if( strcmp( argv[i], "+setname" ) == 0 )
			bDecode = true;
		if( strcmp( argv[i], "-setname" ) == 0 )
			bDecode = false;
		if( strcmp( argv[i], "setname" ) == 0 )
			bDecode = !bDecode;
		if( strcmp( argv[i], "+overwrite" ) == 0 )
			bOverwrite = true;
		if( strcmp( argv[i], "-overwrite" ) == 0 )
			bOverwrite = false;
		if( strcmp( argv[i], "overwrite" ) == 0 )
			bOverwrite = !bOverwrite;
		if( strcmp( argv[i], "-output" ) == 0 )
			output = false;
		if( strcmp( argv[i], "+output" ) == 0 )
			output = true;
		if( strcmp( argv[i], "output" ) == 0 )
			output = !output;
		if( strcmp( argv[i], "-pause" ) == 0 )
			bPaused = false;
		if( strcmp( argv[i], "+pause" ) == 0 )
			bPaused = true;
		if( strcmp( argv[i], "pause" ) == 0 )
			bPaused = !bPaused;
		if( strcmp( argv[i], "-autosave" ) == 0 )
			bAutoSave = false;
		if( strcmp( argv[i], "+autosave" ) == 0 )
			bAutoSave = true;
		if( strcmp( argv[i], "autosave" ) == 0 )
			bAutoSave = !bAutoSave;
		if( strcmp( argv[i], "save" ) == 0 )
			bSaveOptions = true;
	}

	sprintf( szTemp, "Alpha %s", booltostr( bAlphaOnly ) );
	sprintf( szTemp, "%s Autosave %s", szTemp, booltostr( bAutoSave ) );
	sprintf( szTemp, "%s Setname %s",  szTemp, booltostr( bDecode ) );
	sprintf( szTemp, "%s Char %s",  szTemp, booltostr( bDisplayChar ) );
	sprintf( szTemp, "%s Overwrite %s",  szTemp, booltostr( bOverwrite ) );
	sprintf( szTemp, "%s Pause %s",  szTemp, booltostr( bPaused ) );
	sprintf( szTemp, "%s Date %s",  szTemp, booltostr( bShowDate ) );
	sprintf( szTemp, "%s Time %s",  szTemp, booltostr( bShowTime ) );
	sprintf( szTemp, "%s 12h %s",  szTemp, booltostr( b12h ) );
	sprintf( szTemp, "%s Logging %s",  szTemp, booltostr( log_file_set ) );
	sprintf( szTemp, "%s Output %s",  szTemp, booltostr( output ) );

	sprintf( szTemp, "%s   Filter Set: %s",  szTemp, CurrentSection );
	sprintf( szTemp, "%s   Log Name: %s",  szTemp, log_file );

	server->GamePrintInfo( szTemp );

	if( bSaveOptions )
	{
		server->SetHackProfileString( "sniffer", "Options", "Autosave", booltostrb( bAutoSave ) );
		server->SetHackProfileString( "sniffer", "Options", "Overwrite", booltostrb( bOverwrite ) );
		server->SetHackProfileString( "sniffer", "Options", "DisplayChar", booltostrb( bDisplayChar ) );
		server->SetHackProfileString( "sniffer", "Options", "Alpha", booltostrb( bAlphaOnly ) );
		server->SetHackProfileString( "sniffer", "Options", "TagDate", booltostrb( bShowDate ) );
		server->SetHackProfileString( "sniffer", "Options", "TagTime", booltostrb( bShowTime ) );
		server->SetHackProfileString( "sniffer", "Decode", "Decode", booltostrb( bDecode ) );
		
		server->GamePrintInfo( "Options Saved" );
		bSaveOptions = false;
	}

	return true;
}


//////////////////////////////////////////////////////////////////////
// OnGameCommandOutput
// -------------------------------------------------------------------
//////////////////////////////////////////////////////////////////////
BOOL PRIVATE OnGameCommandDecode(char **argv, int argc)			//1-18-2002 - DFlame
{
	int nTemp;
	int i;
	char szTemp[265];
	//char szTemp2[256];
	char *pchTemp;
	char *pszTemp2;

	if( argc < 3 )	// .snifferxp setname r [packet] <name> 
		return false;

	if( strcmp( argv[2], "help" ) == 0 )
		return false;

	if( strcmp( argv[2], "load" ) == 0 )
	{
		for( i=0; i < 256; i++ )
		{
			sprintf( szTemp, "R%.2x", i );
			pszTemp2 = server->GetHackProfileString( "sniffer", "Decode", szTemp );
			if( RecvName[i] != NULL )
				delete RecvName[i];
			if( pszTemp2 != NULL && strcmp( pszTemp2, "" ) != 0 )
			{
				RecvName[i] = new char[ strlen( pszTemp2 ) + 6 ];
				strcpy( RecvName[i], pszTemp2 );
			} else RecvName[i] = NULL;
		}
		for( i=0; i < 256; i++ )
		{
			sprintf( szTemp, "S%.2x", i );
			pszTemp2 = server->GetHackProfileString( "sniffer", "Decode", szTemp );
			if( RecvName[i] != NULL )
				delete SendName[i];
			if( pszTemp2 != NULL && strcmp( pszTemp2, "" ) != 0 )
			{
				SendName[i] = new char[ strlen( pszTemp2 ) + 6 ];
				strcpy( SendName[i], pszTemp2 );
			} else SendName[i] = NULL;
		}
		server->GamePrintInfo( "Packet Names Loaded" );
		return true;
	}

	if( strcmp( argv[2], "save" ) == 0 )
	{
		server->SetHackProfileString( "sniffer", "Decode", "Decode", booltostrb( bDecode ) );
		for( i=0; i < 256; i++ )
		{
			sprintf( szTemp, "R%.2x", i );
			server->SetHackProfileString( "sniffer", "Decode", szTemp, RecvName[i] );
		}
		for( i=0; i < 256; i++ )
		{
			sprintf( szTemp, "S%.2x", i );
			server->SetHackProfileString( "sniffer", "Decode", szTemp, SendName[i] );
		}
		server->GamePrintInfo( "Packet Names Saved" );
		return true;
	}

	nTemp = strtol( argv[3], &pchTemp, 16 );
	if( nTemp < 0x00 || nTemp > 0xff )
	{
		server->GamePrintInfo( "Packet Number out of range < 00 - ff >" );
		return false;
	}

	bool bRecv = false;
	bool bSend = false;

	if ( strcmp( argv[2], "r" ) == 0 )
		bRecv = true;
	if ( strcmp( argv[2], "s" ) == 0 )
		bSend = true;

	if( argc == 4 )
	{
		if( bRecv )
		{
			delete RecvName[nTemp];
			sprintf( szTemp, "Recv Packet %.2x Unset", nTemp );
			server->GamePrintInfo( szTemp );
			return true;
		}
		if( bSend )
		{
			delete SendName[nTemp];
			sprintf( szTemp, "Send Packet %.2x Unset", nTemp );
			server->GamePrintInfo( szTemp );
			return true;
		}
		server->GamePrintInfo( "Specify s - Send, r - Recv" );
		return false;
	}

	sprintf( szTemp, "" );
	for( i=4; i < argc; i++)
	{
		if( strlen( szTemp ) + strlen( argv[i] ) > 255 )
		{
			server->GamePrintInfo( "Packet Name too long." );
			return false;
		}
		sprintf( szTemp, "%s%s", szTemp, argv[i] );
		if( i != argc )
			sprintf( szTemp, "%s ", szTemp );
	}

	if( bRecv )
	{
		if( RecvName[nTemp] != NULL )
			delete RecvName[nTemp];
		RecvName[nTemp] = new char[ strlen( szTemp ) + 6 ];
		if( RecvName[nTemp] == NULL )
		{
			server->GamePrintInfo( "Unable to Allocate Space." );
			return false;
		}
		strcpy( RecvName[nTemp], szTemp );
	}

	if( bSend )
	{
		if( SendName[nTemp] != NULL )
			delete SendName[nTemp];
		SendName[nTemp] = new char[ strlen( szTemp ) + 6 ];
		if( SendName[nTemp] == NULL )
		{
			server->GamePrintInfo( "Unable to Allocate Space." );
			return false;
		}
		strcpy( SendName[nTemp], szTemp );
	}
	server->GamePrintInfo( "Name Set." );
	return true;
}


//////////////////////////////////////////////////////////////////////
// OnGameCommandOutput
// -------------------------------------------------------------------
//////////////////////////////////////////////////////////////////////
BOOL PRIVATE
OnGameCommandOutput (char **argv, int argc)
{
  if (argc != 2)
    return false;
  if ( output )
    {
      output = false;
        server->GamePrintInfo ("Output off.");
    }
  else
    {
      output = true;
      server->GamePrintInfo ("Output on.");
    }
  return true;
}


//////////////////////////////////////////////////////////////////////
// GetTimeString
// -------------------------------------------------------------------
// Convert SYSTEMTIME struct to String
//////////////////////////////////////////////////////////////////////
void PRIVATE GetTimeString( char *pszBuffer, char **argv, int argc )
{
	SYSTEMTIME CurrentSystemTime;
	bool bShowDate = false;
	bool bShowTime = false;
	bool bShow12h = false;
	int i = 0;

	GetLocalTime( &CurrentSystemTime );
	sprintf ( pszBuffer, "" );

	if( argc > 0 )
	{
		for( i = 0; i < argc; i++ )
		{
			if( strcmp( argv[ i ], "date" ) == 0 )
				bShowDate = true;
			if( strcmp( argv[ i ], "time" ) == 0 )
				bShowTime = true;
			if( strcmp( argv[ i ], "12h" ) == 0 )
				bShow12h = true;
		}
	}

	if( bShowDate )
	{
		switch( CurrentSystemTime.wDayOfWeek )
		{
		case 0:
			sprintf( pszBuffer, "Sunday " );
			break;
		case 1:
			sprintf( pszBuffer, "Monday " );
			break;
		case 2:
			sprintf( pszBuffer, "Tuesday " );
			break;
		case 3:
			sprintf( pszBuffer, "Wednesday " );
			break;
		case 4:
			sprintf( pszBuffer, "Thursday " );
			break;
		case 5:
			sprintf( pszBuffer, "Friday " );
			break;
		case 6:
			sprintf( pszBuffer, "Saturday " );
		}

		switch( CurrentSystemTime.wMonth )
		{
		case 1:
			sprintf( pszBuffer, "%sJan ", pszBuffer );
			break;
		case 2:
			sprintf( pszBuffer, "%sFeb ", pszBuffer );
			break;
		case 3:
			sprintf( pszBuffer, "%sMar ", pszBuffer );
			break;
		case 4:
			sprintf( pszBuffer, "%sApr ", pszBuffer );
			break;
		case 5:
			sprintf( pszBuffer, "%sMay ", pszBuffer );
			break;
		case 6:
			sprintf( pszBuffer, "%sJun ", pszBuffer );
			break;
		case 7:
			sprintf( pszBuffer, "%sJul ", pszBuffer );
			break;
		case 8:
			sprintf( pszBuffer, "%sAug ", pszBuffer );
			break;
		case 9:
			sprintf( pszBuffer, "%sSep ", pszBuffer );
			break;
		case 10:
			sprintf( pszBuffer, "%sOct ", pszBuffer );
			break;
		case 11:
			sprintf( pszBuffer, "%sNov ", pszBuffer );
			break;
		case 12:
			sprintf( pszBuffer, "%sDec ", pszBuffer );
		}

		sprintf( pszBuffer, "%s%.2d/%.2d/%.4d", pszBuffer, CurrentSystemTime.wMonth, CurrentSystemTime.wDay,
			CurrentSystemTime.wYear );
	}
	if( bShowDate && bShowTime )
		sprintf( pszBuffer, "%s ", pszBuffer );

	if( bShowTime )
	{
		if( bShow12h && CurrentSystemTime.wHour > 12 )
			sprintf( pszBuffer, "%s%.2d:", pszBuffer, CurrentSystemTime.wHour - 12 );
		else
			sprintf( pszBuffer, "%s%.2d:", pszBuffer, CurrentSystemTime.wHour );

		sprintf( pszBuffer, "%s%.2d:%.2d", pszBuffer, CurrentSystemTime.wMinute, CurrentSystemTime.wSecond );

		if( bShow12h )
		{
			if( CurrentSystemTime.wHour > 12 )
				sprintf( pszBuffer, "%sp", pszBuffer );
			else sprintf( pszBuffer, "%sa", pszBuffer );
		}
	}
}